#!/bin/sh
#
# Dialogue with the user to see which types of tests they want
# to run.  
#

#
# yn ()
#
# Ask a y/n question.  Needs to have QUESTION and DEFAULT_ANS defined
# prior to call.  DEFAULT_ANS must be one of "yes", "no".  Returns 
# answer as "yes", "no" or "default" in 
# ANSWER_YN.
#
yn () {
    ANSWER="foo"
    until [ -z $ANSWER ] || [ $ANSWER = "yes" ] || [ $ANSWER = "y" ] || \
	  [ $ANSWER = "no" ] || [ $ANSWER = "n" ]  ; do {
		echo -n $QUESTION "(yes, no)" "["$DEFAULT_ANS"] "
		read ANSWER
    } ; done

    if [ -z $ANSWER ] ; then {
	ANSWER_YN=$DEFAULT_ANS
    } ; elif [ $ANSWER = "yes" ] || [ $ANSWER = "y" ] ; then {
	ANSWER_YN="yes"
    } ; else {
	ANSWER_YN="no"
    } ; fi
}

#
# The Cerberus test control file
#
echo '
This is the kernel stress test front-end.

To run all the tests, you will need an empty SCSI partition
and an empty IDE partition.  The tests will overwrite any
data in these partitions.  Not to worry though; you will
have a chance to decline these particular tests.

'


CTCS_DRIVER_FILE="./RH-test.tcf"

#
# Flags indicate which tests we want to run
#
NFS_COMPILE="no"
TTCP="no"
IDE="no"
SCSI="no"
MEMORY="no"
FIFO_MMAP="no"
P3_FPU="no"
FS="no"
CRASHME="no"
RAWIO="no"


# Calc total available memory
PMEM_KB=`cat /proc/meminfo | grep '^MemTotal:' | sed 's/^[^0-9]*\([0-9]*\).*/\1/'`
SWAP_KB=`cat /proc/meminfo | grep '^SwapTotal:' | sed 's/^[^0-9]*\([0-9]*\).*/\1/'`


# The maximum amount of memory that testing can possibly use will be: 
# min (RAM * 1.2, RAM + Swap/2) memory.
LOTTA_SWAP_MMB=`echo "( $PMEM_KB + ( $SWAP_KB / 2 ) ) / 1024" | bc -q`
CONSERVATIVE_MMB=`echo "( $PMEM_KB + ( $PMEM_KB / 5 ) ) / 1024" | bc -q`

if [ "$CONSERVATIVE_MMB" -lt "$LOTTA_SWAP_MMB" ] ; then
    MMB="$CONSERVATIVE_MMB"
else
    MMB="$LOTTA_SWAP_MMB"
fi


# Reserve 256 MB for the kernel and sys processes on enterprise kernel
# Reserve 64 MB on non-enterprise kernels
if [ `uname -r | grep "enterprise"` ] ; then
	RESERVE_KERNEL=256
else
	RESERVE_KERNEL=64
fi


# Give the user a chance to reserve some additional memory.
echo "How much extra memory (in MB) beyond the defaults do you want to reserve? [0] "
read RESERVE_EXTRA
[ $RESERVE_EXTRA ] || RESERVE_EXTRA=0


# Finally, calculate maximum memory Cerberus will attempt to use.
MEM_CEILING=`echo "$MMB - $RESERVE_KERNEL - $RESERVE_EXTRA" | bc -q`
echo "Calculating maximum allocable memory =" $MEM_CEILING "MB"

#These variables define the amount of memory in MB to reserve for each test.
SCSI_MEM=8
TTCP_MEM=1
NFS_COMPILE_MEM=48
IDE_MEM=10
FIFO_MMAP_MEM=206
P3_FPU_MEM=10
FS_MEM=352
CRASHME_MEM=1
RAWIO_MEM=50

#
# SCSI dialogue
#
QUESTION="Do you want to run the destructive SCSI tests?"; DEFAULT_ANS="no"
yn
SCSI=$ANSWER_YN
if [ $SCSI = "yes" ] ; then {
    echo "Enter name of a SCSI partition that can be overwritten."
    echo -n "   (You can just type <return> here to skip this test.) "
    read SCSI_PARTITION
    if [ -z $SCSI_PARTITION ] ; then {
	SCSI="no"
	echo "SCSI test will be skipped."
    } ; else {
	echo "SCSI test will write to" $SCSI_PARTITION"."
	
    } ; fi
} ; else {
    echo "SCSI test will be skipped."
} ; fi

echo

#
# TTCP dialogue
#
QUESTION="Do you want to test TCP/IP?"; DEFAULT_ANS="yes"; yn
TTCP=$ANSWER_YN
if [ $TTCP = "yes" ] ; then {
    echo "Enter hostname or IP number of this host:"
    echo -n "  (You can just type <return> here to skip this test.) "
    read LOCAL_HOST
    if [ -z $LOCAL_HOST ] ; then {
	echo "TCP/IP test will be skipped.";
	TTCP="no"
    } ; else {
	    echo "Enter hostname or IP number of remote host:"
	    echo -n "  (You can just type <return> here to skip this test.) "
	    read REMOTE_HOST
	    if [ -z $REMOTE_HOST ] ; then {
		echo "TCP/IP test will be skipped.";
		TTCP="no"
	    } ; else {
		echo "TCP/IP will run between" $LOCAL_HOST "and" $REMOTE_HOST"."
	    } ; fi
    } ; fi
} ; else {
   echo "TCP/IP test will be skipped."
} ; fi

echo

QUESTION="Do you want to run the NFS-compile test?"; DEFAULT_ANS="yes"; yn
NFS_COMPILE=$ANSWER_YN
if [ $NFS_COMPILE = "yes" ] ; then {
    echo "NFS compile will be run."
} ; else {
    echo "NFS compile will be skipped."
} ; fi

echo

#
# IDE dialogue
#
QUESTION="Do you want to run the destructive IDE tests?"; DEFAULT_ANS="no"; yn
IDE=$ANSWER_YN
if [ $IDE = "yes" ] ; then {
    echo "Enter name of an IDE disk (device, not partition)"
    echo "	that can be overwritten."
    echo -n "   (You can just type <return> here to skip this test.) "
    read IDE_PARTITION
    if [ -z $IDE_PARTITION ] ; then {
	IDE="no"
	echo "IDE test will be skipped."
    } ; else {
	echo "IDE test will write to" $IDE_PARTITION"."
    } ; fi
} ; else {
    echo "IDE test will be skipped."
} ; fi

echo

QUESTION="Do you want to run memory tests?"; DEFAULT_ANS="yes"; yn
MEMORY=$ANSWER_YN
if [ $MEMORY = "yes" ] ; then {
    echo "Memory test will be run."
} ; else {
    echo "Memory test will be skipped."
} ; fi

echo

QUESTION="Do you want to test FIFO's and mmap?"; DEFAULT_ANS="yes"; yn
FIFO_MMAP=$ANSWER_YN
if [ $FIFO_MMAP = "yes" ] ; then {
    echo "FIFO/mmap test will be run."
} ; else {
    echo "FIFO/mmap test will be skipped."
} ; fi

echo

QUESTION="Do you want to run the P3 FP registers test?"; DEFAULT_ANS="yes"; yn
P3_FPU=$ANSWER_YN
if [ $P3_FPU = "yes" ] ; then {
    echo "P3 FP registers test will be run."
} ; else {
    echo "P3 FP registers test will be skipped."
} ; fi

echo
	    
QUESTION="Do you want to run the file system ops test?"; DEFAULT_ANS="yes"; yn
FS=$ANSWER_YN
if [ $FS = "yes" ] ; then {
    echo "File system ops test will be run."
} ; else {
    echo "File system ops test will be skipped."
} ; fi

echo

QUESTION="Do you want to run crashme?"; DEFAULT_ANS="yes"; yn
CRASHME=$ANSWER_YN
if [ $CRASHME = "yes" ] ; then {
    echo "Crashme will be run."
} ; else {
    echo "Crashme will be skipped."
} ; fi

echo

QUESTION="Do you want to run the raw io test?"; DEFAULT_ANS="no"; yn
RAWIO="$ANSWER_YN"
if [ "$RAWIO" = "yes" ] ; then {
    echo "Enter name of raw device "
    echo "  (You can just type <return> here to skip this test.) "
    read RAW_D
    if [ -n "$RAW_D" ] ; then {
	echo "Enter name of block device "
	echo "  (You can just type <return> here to skip this test.) "
	read BLOCK_D
	if [ -n "$BLOCK_D" ] ; then {
	    echo
	    echo "Raw IO test will be run."
	} ; else {
	    echo "Raw IO test will be skipped."
        } ; fi
    } ; else {
	echo "Raw IO test will be skipped."
    } ; fi
} ; else {
    echo "Raw IO test will not be run."
} ; fi

echo
# 
# Create the driver file
#
rm -rf $CTCS_DRIVER_FILE >> /dev/null 2>&1
cat > $CTCS_DRIVER_FILE <<EOF
#
# This is the ctcs driver file.  This file is auto-created.
#
set verbose 1
EOF

#
# This is a dummy test which just sleeps for a minute and then
# writes /proc/ksyms to a safe place.
#
cat >> $CTCS_DRIVER_FILE<<EOF
bg 1 SAVE-STATE save_state.sh
EOF

if [ $SCSI = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE <<EOF
bg 1 SCSI-TEST iotest30_driver.sh $SCSI_PARTITION
EOF
MEM_CEILING=`expr "$MEM_CEILING" - "$SCSI_MEM"`
} ; fi

if [ $NFS_COMPILE = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE <<EOF
bg 4 NFS-COMPILE nfstest.sh
EOF
MEM_CEILING=`expr "$MEM_CEILING" - "$NFS_COMPILE_MEM"`
} ; fi

if [ $TTCP = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 1024 TTCP ttcp_driver.sh $LOCAL_HOST $REMOTE_HOST
EOF
MEM_CEILING=`expr "$MEM_CEILING" - "$TTCP_MEM"`
} ; fi

if [ $IDE = "yes" ] ; then {
BLOCKRDTST_ARGS=`./runin/blockrdtst-info "$IDE_PARTITION"`
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 4 IDE blockrdtst $BLOCKRDTST_ARGS
EOF
MEM_CEILING=`expr "$MEM_CEILING" - "$IDE_MEM"`
} ; fi


if [ $FIFO_MMAP = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 256 FIFOS_MMAP dt_driver.sh
EOF
MEM_CEILING=`expr "$MEM_CEILING" - "$FIFO_MMAP_MEM"`
} ; fi

if [ $P3_FPU = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 256 P3-FPU P3-stress.sh
EOF
MEM_CEILING=`expr "$MEM_CEILING" - "$P3_FPU_MEM"`
} ; fi

if [ $FS = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 64 FS fs-test-driver.sh
EOF
MEM_CEILING=`expr "$MEM_CEILING" - "$FS_MEM"`
} ; fi

if [ $CRASHME = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 256 CRASHME crashme_driver.sh
EOF
MEM_CEILING=`expr "$MEM_CEILING" - "$CRASHME_MEM"`
} ; fi

if [ $RAWIO = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 1024 RAWIO rawio-driver.sh $RAW_D $BLOCK_D
EOF
MEM_CEILING=`expr "$MEM_CEILING" - "$RAWIO_MEM"`
} ; fi

if [ $MEMORY = "yes" ] ; then {
   
    if [ "$MEM_CEILING" -le 0 ] ; then {
	echo "There isn't enough memory to run the memory test,"
	echo "so it will be skipped."
	MEMORY="no"
    } ; else {

	NMEMTEST=0
	while [ $MEM_CEILING -gt 0 ] ; do {
	    if [ $MEM_CEILING -lt 1800 ] ; then {
		TESTUSAGE=$MEM_CEILING
		MEM_CEILING=0
	    } ; else {
		TESTUSAGE=1800
		MEM_CEILING=`expr $MEM_CEILING - 1800`
	    } ; fi
	    cat >> $CTCS_DRIVER_FILE<<EOF
bg 64 MEMORY$NMEMTEST memtst 32 $TESTUSAGE
EOF
	    NMEMTEST=`expr $NMEMTEST + 1`
	} ; done

    } ; fi

} ; fi


cat >> $CTCS_DRIVER_FILE<<EOF
wait
exit
EOF

echo; echo; echo "I'm about the run the following tests:"
if [ $SCSI = "yes" ] ; then { echo "  SCSI"; } ; fi
if [ $TTCP = "yes" ] ; then { echo "  TCP/IP"; } ; fi
if [ $NFS_COMPILE = "yes" ] ; then { echo "  NFS compile"; } ; fi
if [ $IDE = "yes" ] ; then { echo "  IDE"; } ; fi
if [ $MEMORY = "yes" ] ; then { echo "  Memory"; } ; fi
if [ $FIFO_MMAP = "yes" ] ; then { echo "  FIFO's/mmap"; } ; fi
if [ $P3_FPU = "yes" ] ; then { echo "  P3 FP registers"; } ; fi
if [ $FS = "yes" ] ; then { echo "  File system ops"; } ; fi
if [ $CRASHME = "yes" ] ; then { echo "  Crashme"; } ; fi
if [ $RAWIO = "yes" ] ; then { echo "  Raw IO"; } ; fi
echo

QUESTION="Proceed?"; DEFAULT_ANS="yes"; yn
PROCEED=$ANSWER_YN
if [ $PROCEED = "no" ] ; then {
    exit 0
} ; fi

echo "

OK, I'm ready to run the tests.  You can bail out at any point 
with <ctrl>-c.
"

setterm -blank 0

exec ./run RH-test.tcf
