#!/bin/sh

DEBUG=0

function fail () {
    rm -rf $TEMP_DIR 2>&1 >> /dev/null ;
    echo $ERR ;
    exit $ERR_NUM ;
}

# 1 MB + 1
FILESIZE=1048577

RANDOM_FILES=""
RVALS="0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0"
NON_RANDOM_FILES="nr0 nr1 nr2 nr3 nr4 nr5"
ALL_FILES=

#SOURCE_DIR=P3
TEMP_DIR=/tmp/P3-$$
P3_dir=`pwd`/P3

trap "rm -rf $TEMP_DIR >> /dev/null 2>&1; exit -1" SIGINT SIGHUP SIGTERM

mkdir $TEMP_DIR >> /dev/null 2>&1
[ $? -eq 0 ] || { ERR="Can't make temp dir $TEMP_DIR." ; ERR_NUM=$? ; fail ; }

# create random data files
for I in $RVALS ; do {
    $P3_dir/makedatafiles -r $I -s $FILESIZE -o $TEMP_DIR/r$I
    [ $? -eq 0 ] || { ERR_NUM=$?;
		      rm -rf $TEMPDIR >> /dev/null 2>&1 ; 
		      ERR="Can't make test file $TEMP_DIR/r$I." ;
		      fail ; 
		      }
    ALL_FILES=`echo "$ALL_FILES" "$TEMP_DIR/r$I"`
} ; done

#create nonrandom data files
for I in $NON_RANDOM_FILES ; do {
    $P3_dir/makedatafiles -r 0.0 -s $FILESIZE -o $TEMP_DIR/$I
    [ $? -eq 0 ] || { ERR_NUM=$? ;
		      rm -rf $TEMPDIR >> /dev/null 2>&1 ; 
		      ERR="Can't make test file $TEMP_DIR/$I." ;
		      fail ; 
		      }
    ALL_FILES=`echo "$ALL_FILES" "$TEMP_DIR/$I"`
} ; done


cd $TEMP_DIR >> /dev/null 2>&1
[ $? -eq 0 ] || 
    { ERR_NUM=$? ; 
      rm -rf $TEMP_DIR >> /dev/null 2>&1 ; 
      ERR="Can't cd to $TEMP_DIR." ; fail ; }

for i in $ALL_FILES; do {
    if [ -e $i ] ; then {
	if [ $DEBUG -eq 0 ] ; then {
	    echo "Starting file $i..."
	} ; fi
	bzip2 -9 $i & sleep 1
    }
    else {
	ERR="Can't find file $i."
	ERR_NUM=-1
	fail
    } ; fi
} ; done


if [ $DEBUG -eq 0 ] ; then {
    echo "waiting"
} ; fi

wait

if [ $DEBUG -eq 0 ] ; then {
    echo "resuming"
} ; fi


for i in $ALL_FILES ; do {
    if [ $DEBUG -eq 0 ] ; then {
	echo "testing $i.bz2..."
    } ; fi
    bzip2 -t $i.bz2
    [ $? -eq 0 ] || { ERR="bzip found corrupted file $i." ; ERR_NUM=$? ; fail ; }
} ; done


for i in $ALL_FILES ; do {
    if [ $DEBUG -eq 0 ] ; then {
	echo "bunziping $i.bz2..."
    } ; fi
    bunzip2 $i.bz2
    [ $? -eq 0 ] || { ERR="Can't bunzip $i.bz2." ; ERR_NUM=$? ; fail ; }
} ; done

rm -rf $TEMP_DIR >> /dev/null 2>&1
[ $? -eq 0 ] || { ERR="Can't clean up temporary dir $TEMP_DIR." ; ERR_NUM=$? ; fail ; }

trap "killall mat_mult; exit 0" SIGINT SIGHUP SIGTERM

$P3_dir/mat_mult
[ $? -eq 0 ] || { ERR="Mat Mult failed." ; ERR_NUM=$? ; fail ; }

exit 0

