#!/bin/bash
#
# A driver for crashme.
#
# Crashme usage: 
#    crashme <nbytes> <srand> <ntrys> <time> <verbosity, >> = more>

#
# Must be SIGKILL here.  Crashme catches most everything else.
#
cleanup() {

    /usr/bin/killall -q -KILL crashme
    /usr/sbin/userdel "$CRASHER_NAME"
    [ $? -eq 0 ] || { echo "Can't delete crashme user." ; \
	exit 1 ; }
    exit 0
}

ITERATIONS=5
BASH=/bin/bash

# This is the info for the fictional user who will execute crashme
#
# We start looking for an unused uid at $START_UID
#
CRASHER_NAME="crashme"
CRASHER_PASSWD='$1$F1o51gmr$xKg3eYEDSgIvZx5WdKOIc.'
CRASHER_UID=40000 

EXISTS=`grep "$CRASHER_UID" /etc/passwd`
while [ "$EXISTS" ] ; do
    CRASHER_UID=`expr $CRASHER_UID + 1`
    EXISTS=`grep $CRASHER_UID /etc/passwd`
done

# If the crashme_user already exists (for example, if the kernel
# oopsed on a previous test run) we don't need to create a crashme
# user.

CRASHER_EXISTS=`grep "$CRASHER_NAME" /etc/passwd`
if [ -z "$CRASHER_EXISTS" ] ; then

    /usr/sbin/useradd -d /tmp -p "$CRASHER_PASSWD" -s /bin/bash \
	-u "$CRASHER_UID" "$CRASHER_NAME"
    [ $? -eq 0 ] || { echo "Can't create crashme user with uid $CRASHER_UID" ; \
	exit 1 ; }

fi

trap cleanup SIGINT SIGHUP SIGTERM

while [ "$ITERATIONS" -gt 0 ] ; do
    SEED=`./bin/random`
    FRAGMENT_SIZE=`echo 2 ^ \( $ITERATIONS + 10 \) | bc -q`
    su $CRASHER_NAME -c './bin/crashme "$FRAGMENT_SIZE" "$SEED" 100 0:0:10 2' &
    ITERATIONS=`expr "$ITERATIONS" - 1`
done

wait

cleanup

