#!/bin/sh

#
# A driver for the dt program
#

function cleanup () {
    rm -f $TEMP_FIFO $TEMP_FILE >> /dev/null 2>&1
}

trap "/usr/bin/killall -q -TERM dt; cleanup; exit 0" SIGINT SIGHUP SIGTERM

#
# FIFO test.  Make a FIFO and cram lots of stuff through it.  
#

DT_HOME="./dt.d"
TEMP_DIR=/tmp
TEMP_FIFO=$TEMP_DIR/FIFO.tmp.$$
mkfifo $TEMP_FIFO >> /dev/null 2>&1
[ $? = 0 ] || { echo "Can't make FIFO $TEMP_FIFO."; exit -1 ; }

PAGESIZE=`./getpagesize` # dt will croak if buffer size > page size
[ $? = 0 ] || { echo "Can't get system page size!"; cleanup; exit -1 ; }
BLOCKSIZE=$PAGESIZE

#
# First, let's see if we can croak the FIFO by shoving > 2^32 bits
# through it.
#

cd "$DT_HOME"

./dt of=$TEMP_FIFO bs=$PAGESIZE limit=1g+1m+1k+1 enable=loopback &
wait $!

RES=$?
[ "$RES" = 0 ] || { echo "Dt crapped out on big FIFO test."; cleanup; exit "$RES" ; }

# 
# OK, now we'll just loop through trying this with a variety of different
# block sizes from $PAGESIZE .. 1
#
while [ $BLOCKSIZE -gt 0 ] ; do {
    ./dt of=$TEMP_FIFO bs=$BLOCKSIZE limit=1M enable=loopback
    [ $? = 0 ] || { echo "Dt crapped out on FIFO test."; cleanup; exit $? ; }
    BLOCKSIZE=`expr $BLOCKSIZE / 4`
} done

#
# Mmap'd files test
#
# Read and verify blocks from an mmap'd file.  File size ranges from
# 1M to 256M
#

FILESIZE=1 # Size in MB
MAX_FILESIZE=`echo "2^8" | bc -q` # Should give us 2^8 MB = 256 MB max.

TEMP_FILE=`mktemp /tmp/dt.tmp.XXXXXXX`
[ $? = 0 ] || \
    { echo "Can't create temp file for dt mmap test."; cleanup; exit -1 ; }

while [ $FILESIZE -le $MAX_FILESIZE ] ; do {
    ./dt of=$TEMP_FILE bs=$PAGESIZE limit="$FILESIZE"m enable=mmap,verify &
    wait $!
    RES=$?
    [ "$RES" = 0 ] || { echo "Dt failed during mmap test."; cleanup; exit "$RES" ; }
    FILESIZE=`expr $FILESIZE \* 4`
} done

cleanup
exit 0



