#!/bin/sh
#
#This test creates a deeply nested series of directories and then
#drops a dummy file into the bottom.  We then attempt to remove the
#directory trees from the bottom up, first with a file in the
#dir  and then with an empty dir.
#
#Usage dir.sh <working_dir>
#

fatal() {
    echo "$ERR";
    exit -1;
}

[ $1 ] || { ERR="Usage: dir.sh <working_dir>" ; fatal ; }

DEPTH=500
WORKING_DIR=$1

[ -d "$WORKING_DIR" ] || mkdir $WORKING_DIR || 
    { ERR="Can't make $WORKING_DIR." ; fatal; }

cd "$WORKING_DIR" || { ERR="Can't cd to $WORKING_DIR." ; fatal; }

#
# Make subdirs $DEPTH deep and descend into each one.
#
I=$DEPTH;
while [ `expr $I` -ne 0 ] ; do {
    mkdir "dir"$I || { ERR="Can't mkdir dir$I." ; fatal; }
    cd "dir$I" || { ERR="Can't cd to dir$I." ; fatal; }
    I=`expr $I - 1` ;
} ; done

#
# Create a dummy file
#
cat<<EOF > dummy_file
This is some text.
EOF

[ -f dummy_file ] || { ERR="Didn't create dummy_file." ; fatal; }

#
# Now, try to remove the most deeply nested directory (which is non-empty.)
# When this fails, move the dummy file up on level, remove the (now empty)
# directory, and repeat the process until the created dirs are all
# deleted.
#
I=1
while [ `expr $I` -ne `expr $DEPTH + 1` ] ; do {

# cd to parent and try to remove non-empty child dir.
  cd .. || { ERR="Can't cd to parent of `pwd`." ; fatal; }
  rmdir "dir$I"  >> /dev/null 2>&1
  [ $? = 1 ]  || { ERR="Removed non-empty dir dir$I." ; fatal; }

# move the file to . and retry the removal, which should succeed
  mv "dir$I"/dummy_file . || { ERR="Can't move dummy_file to dir$I." ; fatal; }
  rmdir "dir$I"  >> /dev/null 2>&1
  [ $? = 0 ] || { ERR="Could not rmdir dir$I." ; fatal; }

  I=`expr $I + 1` ;
} ; done

rm dummy_file || { ERR="Can't remove dummy_file." ; fatal; }

exit 0