#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#define MAXFNAME 128
#define SEED 12345
#define LOW ((unsigned long long) 0x100000000ULL)
#define HIGH ((unsigned long long) 0xFFFFFFFFFFFFFFFEULL)
#define MAX_ERR_MSG 1024

/* 
 * Return a random u_long in the interval [low, high] 
 */
loff_t randint (unsigned long long low, unsigned long long high)
{
  unsigned long long normalized;

  normalized = (unsigned long long) ( (float) rand() / 
		       ( (float) RAND_MAX / (float) (high - low + 1) ) );
  return (normalized + low);
}


int main (int argc, char *argv[])

{

  char *fname;
  int iterations, fd, i;
  off_t offset;

  if (argc != 3) {
    printf ("Usage: large_truncate [file_name] [iterations]\n");
    exit (1);
  }

  if ( (fname = (char *) malloc (MAXFNAME)) == NULL) {
    printf ("Malloc failed.\n");
    exit (1);
  }
  
  strncpy (fname, argv[1], (size_t) MAXFNAME);

  iterations = strtol (argv[2], NULL, 10);

  srand (SEED);

  for (i = 0; i < iterations; i++) {

    if ( (fd = open64(fname, O_RDWR | O_CREAT | O_NOFOLLOW | O_LARGEFILE, 
		      S_IRWXU)) == -1) {
      perror ("Open64");
      exit (1);
    }

    offset = randint (LOW, HIGH);

    /* printf ("Offset (0x%lx, 0x%lx) = 0x%lx\n", LOW, HIGH, offset); */

    if (ftruncate64 (fd, offset) == -1) {
	char* errmsg = (char *) malloc (MAX_ERR_MSG);
	sprintf (errmsg, "Ftruncate64, offset=%ld", offset);
	perror (errmsg);
	free (errmsg);
    }

    if (close (fd) == -1) {
      perror ("Close");
      exit (1);
    }

    if (unlink (fname) == -1) {
      perror ("Unlink");
      exit (1);
    }

  }

  exit (0);

}
		    
