#!/bin/sh

fatal() {
    echo "$ERR"
    exit -1
}

[ $1 ] || { ERR="Usage: slinks.sh <working_dir>." ; fatal ; }

BASE_DIR=$1
DEPTH=5
ITERATIONS=100

J=1
while [ "$ITERATIONS" -ge "$J" ] ; do {

    HOME_DIR="$BASE_DIR"/slink-$$-$J
    ORIG_FILE="$HOME_DIR"/slink-targ

    [ -d "$HOME_DIR" ] || 
	{ mkdir "$HOME_DIR"  ; } || 
	{ ERR="Can't make $HOME_DIR." ; fatal ; }

    cd "$HOME_DIR" || { ERR="Can't cd to $HOME_DIR." ; fatal ; }

    cat <<EOF > $ORIG_FILE
This is the content of the original file.
We really need more data than this.
It should be added later.
abcdefghijklmnopqrstuvwxyz
ABCDEFGHIJKLMNOPQRSTUVWXYZ
123456890-=
!@#$%^*_+
\|;':",./<>?~ '
blarg murch grume rops quvt cest larpy clur flunp chirl abeds
sdeba rlihc pnuf rulc sporn emurg chrum gralb ilipity zertoid
flong triben plesto gressy blenkier rin moxilivy safs wollier
bern troo bleef porify triffel blabity nogify hurn lenuary ar
This is the content of the original file.
We really need more data than this.
It should be added later.
abcdefghijklmnopqrstuvwxyz
ABCDEFGHIJKLMNOPQRSTUVWXYZ
123456890-=
!@#$%^*_+
\|;':",./<>?~ '
blarg murch grume rops quvt cest larpy clur flunp chirl abeds
sdeba rlihc pnuf rulc sporn emurg chrum gralb ilipity zertoid
flong triben plesto gressy blenkier rin moxilivy safs wollier
bern troo bleef porify triffel blabity nogify hurn lenuary ar
EOF
    echo $$  >> $ORIG_FILE

    CHECK_SUM=`md5sum $ORIG_FILE | cut -f1 -d' '`

    ln -s $ORIG_FILE slinkA0 
    [ $? -eq 0 ] || 
	{ ERR="Can't make symlink $HOME_DIR/slink0 to $ORIG_FILE." ; fatal ; }

    I=1
    while [ $I -ne $DEPTH ] ; do {

	mkdir d"$I" 
	[ $? -eq 0 ] || { ERR="Can't make directory d$I." ; fatal ; }
	cd d"$I"
	[ $? -eq 0 ] || { ERR="Can't cd to directory d$I." ; fatal ; }

	SOURCE1=slinkA"$I"
	SOURCE2=slinkB"$I"
	TARGET1=../slinkA`expr $I - 1`

	ln -s $TARGET1 $SOURCE1   
	[ $? -eq 0 ] || 
	    { ERR="Can't link $SOURCE1 to $TARGET1." ; fatal ; }

	ln -s $HOME_DIR/slinkA0 $SOURCE2 
	[ $? -eq 0 ] ||
	    { ERR="Can't link to $HOME_DIR/slinkA0." ; fatal ; }

	RESULT1=`md5sum $SOURCE1 | cut -f1 -d' '`
	[ "$RESULT1" = "$CHECK_SUM" ] ||
	    { ERR="Wrong checksum for $SOURCE1." ; fatal ; }

	RESULT2=`md5sum $SOURCE2 | cut -f1 -d' '`
	[ "$RESULT2" =  "$CHECK_SUM" ] ||
	    { ERR="Wrong checksum for $SOURCE2." ; fatal ; }

	

	I=`expr "$I" + 1`

     } ; done

#One more test, remove orig file, symlink access should fail.
    rm -rf $ORIG_FILE 
    [ $? -eq 0 ] || { ERR="Can't remove $ORIG_FILE." ; fatal ; }
    cat "$HOME_DIR"/slinkA0 >> /dev/null 2&>1
    [ $? -ne 0 ] || 
	{ ERR="Yikes!  Accessed deleted file through symlink $HOME_DIR/slinkA0." ; fatal ; }

    J=`expr "$J" + 1` 
    rm -rf $HOME_DIR 

} ; done

exit 0

