#!/bin/sh
#
# A file to drive the IOtest30 series of tests.
#
IOTEST_HOME=`pwd`
DEVICE=$1
MAX_PROCESSES=12
#ITERATIONS=1000

[ -z $DEVICE ] && { echo "You must specify a device for IOtest30" ; exit 2 ; }

cd "$IOTEST_HOME" || { echo "Can't cd to $IOTEST_HOME." ; exit 2 ; }
#
# Block read benchmark
#
./IOtest -d $DEVICE -r -p $MAX_PROCESSES  & 
TEST_PID=$!

# IOtest isn't catching any sigs, and the actual tests get
# forked off to run, so we have to use /bin/kill here.
#
trap "/usr/bin/killall -q -TERM IOtest; exit 2" SIGINT SIGHUP SIGTERM

wait $TEST_PID
[ $? -eq 0 ] || exit $?

#
# Block write benchmark
#
echo "yes" | ./IOtest -d $DEVICE -w  -p $MAX_PROCESSES  &
TEST_PID=$!
wait $TEST_PID
[ $? -eq 0 ] || exit $?

#
# Random read/write test.  Run once with 5%/95% r/w, once with
# 50/50 and once with 95/5.
#
echo "yes" | ./IOtest -d $DEVICE -a 5  -p $MAX_PROCESSES  &
TEST_PID=$!
wait $TEST_PID
[ $? -eq 0 ] || exit $?

echo "yes" | ./IOtest -d $DEVICE -a 50  -p $MAX_PROCESSES  &
TEST_PID=$!
wait $TEST_PID
[ $? -eq 0 ] || exit $?

echo "yes" | ./IOtest -d $DEVICE -a 95  -p $MAX_PROCESSES  &
TEST_PID=$!
wait $TEST_PID
[ $? -eq 0 ] || exit $?

exit 0
