/* 2001 Andrea Arcangeli <andrea@suse.de>         */
/*                                                */
/* 12/04/01 - Simple changes to make it live      */
/*            happily in Cerberus land.           */
/*            Bob Matthews <bmatthews@redhat.com> */

#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <asm/page.h>

#define MB (1024*1024)
#define BUFSIZE (50*MB)

#define MAX_DEVICE_NAME 32
#define MAX_FILE_NAME 128
#define MAX_SYS_COMMAND 128
#define MAX_ERR_MSG 32

int main(int argc, char **argv)
{
        int fd, size, ret;
        int filemap;
        char * buf, * end, * tmp;

	char raw_d[MAX_DEVICE_NAME], block_d[MAX_DEVICE_NAME];
	char data_file[MAX_FILE_NAME], sys_command[MAX_SYS_COMMAND];
	char err_msg[MAX_ERR_MSG];

	if (argc != 4) {
	  printf ("Usage: raw_io <raw interface> <block device> <tmp file>\n");
	  exit (1);
	}

	strncpy (raw_d, argv[1], MAX_DEVICE_NAME);
	strncpy (block_d, argv[2], MAX_DEVICE_NAME);
	strncpy (data_file, argv[3], MAX_FILE_NAME);
	
	sprintf (sys_command, "raw %s %s\n", raw_d, block_d);
	if (system (sys_command)) {
	  perror ("system");
	  exit(1);
	}

        printf("Opening %s\n", raw_d);
        fd = open(raw_d, O_RDWR);
        if (fd < 0) {
	  sprintf (err_msg, "open %s", raw_d);
          perror (err_msg);
	  exit(1);
	}

        printf("Allocating %dMB of memory\n", BUFSIZE/MB);
        buf = (char *) malloc(BUFSIZE);
        if (buf < 0) {
                perror("malloc");
		exit (1);
	}
        end = (char *) ((unsigned long) (buf + BUFSIZE) & PAGE_MASK);
        buf = (char *) ((unsigned long)(buf + ~PAGE_MASK) & PAGE_MASK);

        printf("Mapping %dMB of memory\n", BUFSIZE/MB);
	
        filemap = open(data_file, O_RDWR|O_TRUNC|O_CREAT, 0644);
        if (filemap < 0) {
	  sprintf (err_msg, "open %s", data_file);
	  perror(err_msg);
	  exit(1);
	}

        {
	  int i;
	  char buf[4096];
	  for (i = 0; i < BUFSIZE; i += 4096)
	    write(filemap, &buf, 4096);
        }

        if (ftruncate(filemap, BUFSIZE)) {
	  sprintf (err_msg, "ftruncate %s", data_file);
	  perror (err_msg);
	  exit (1);
	}

        buf = mmap(0, BUFSIZE, PROT_READ|PROT_WRITE, MAP_SHARED, filemap, 0);
        if ((long) buf < 0) {
	  perror("mmap");
	  exit(1);
	}
        
	if ((unsigned long) buf & ~PAGE_MASK) {
	  perror("mmap misaligned");
	  exit(1);
	}

        end = buf + BUFSIZE;
        size = end - buf;

        printf("Reading from %s\n", raw_d);

        ret = read(fd, buf, size);
        if (ret < 0) {
	  sprintf (err_msg, "read %s", raw_d);
	  perror(err_msg);
	  exit(1);
	}
        if (ret != size)
                fprintf(stderr, "read only %d of %d bytes\n", ret, size);

        printf("Writing data to %s\n", raw_d);
        if (lseek(fd, 0, SEEK_SET) < 0) {
                perror("lseek");
		exit(1);
	}

        ret = write(fd, buf, size);
        if (ret < 0) {
	  sprintf (err_msg, "read %s", raw_d);
	  perror("read /dev/raw1");
	  exit(1);
	}
        
	if (ret != size)
	  fprintf(stderr, "write only %d of %d bytes\n", ret, size);

	if (close(filemap)) {
	  sprintf (err_msg, "close %s", data_file);
	  perror (err_msg);
	  exit (1);
	}

	if (unlink(data_file)) {
	  sprintf (err_msg, "unlink %s", data_file);
	  perror (err_msg);
	  exit (1);
	}
}

